
/**
 * Klasse Vigenere.
 * 
 * @author (claus albowski) 
 * @version (Jan'06)
 */
public class Vigenere 
implements VigenereInterface
{
    // Instanzvariable
    String original, code;
    String key;
    int keyLength, codeTextLength;
    boolean originalTextEingegeben, codeTextEingegeben, 
            schluesselEingegeben, istCodiert, istDecodiert;
    int[] haeufigkeiten;
    double kappa, laenge;

    /**
     * Konstruktor fr Objekte der Klasse Caesar
     */
    public Vigenere()
    {
        // Instanzvariable initialisieren
        original=code=null; 
        key = "A";
        keyLength=1;
        codeTextLength=0;
        haeufigkeiten=null;
        kappa=laenge=0.0;
        originalTextEingegeben=codeTextEingegeben=schluesselEingegeben
                  =istCodiert=istDecodiert=false;
    }
    // ffentliche Methoden:
    
    
    /**
     * gibt jeweils den bearbeiteten Text aus.
     */
    public String zeigeText(){
        if (istDecodiert) return original;
        else if (istCodiert) return code;
        else return "Keine Bearbeitung!";
    }
    
    /**
     * setzt den Originaltext.
     */
    public void setzeOriginalText(String text){
        original=text;
        code = null;
        originalTextEingegeben=true;
        istCodiert=istDecodiert=false;
    }

    /**
     * setzt den Codetext und berechnet die Analysewerte neu.
     */
    public void setzeCodetext(String text){
        code=text;
        original=null;
        codeTextEingegeben=true;
        istCodiert=istDecodiert=false;
        analyse();
    }

    /**
     * setzt den Schluessel.
     */
    public void setzeSchluessel(String schluessel){
        key=schluessel;
        keyLength=key.length();
        schluesselEingegeben=true;
    }

    /**
     * verschluesselt den eingegebenen Text und berechnet die Analysewerte neu.
     */
    public void verschluessele(){
        if (originalTextEingegeben){
            code = verschluessele(original, false);
            istCodiert=true;
            analyse();
        }
    }

    /**
     * entschluesselt den eingegebenen Text.
     */
    public void entschluessele(){
        if (codeTextEingegeben){
            original = verschluessele(code, true);
            istDecodiert=true;
        }
    }
    
    /**
     * prft, ob ein Zeichen Grobuchstabe ist.
     */
    public boolean istGrossBuchstabe(Character c){
        return !((c.compareTo('A')<0)||(c.compareTo('Z')>0));
        // interessanterweise wird  zu ss vorher! konvertiert!
    }
    
    /**
     * gibt zum aktuellen Codetext die Hufigkeit der Buchstaben des Codes in einem Array zurck.
     */  
    public int[] gibHaeufigkeiten(){
        if (haeufigkeiten!=null) return haeufigkeiten; else return new int[26];
    }
    
    /**
     * gibt zum aktuellen Codetext die Hufigkeit der Buchstaben des Codes 
     * an der index-ten Schlsselposition bei laenge Schlssellnge in einem Array zurck.
     */  
    public int[] gibHaeufigkeiten(int index, int laenge){
        if (code!=null){
            StringBuffer temp=new StringBuffer();
            for (int i=index;i<codeTextLength;i+=laenge) temp.append(code.charAt(i));
            return haeufigkeiten(temp.toString());
        }
        else return new int[26];
    }
    
    /**
     * gibt zum aktuellen Codetext kappa zurck.
     */
    public double gibKappa(){
        return kappa;
    }
    
    /**
     * gibt zum aktuellen Codetext die vermutliche Schlssellnge zurck.
     */
    public double gibLaenge(){
        return laenge;
    }
    
    
    // private Methoden:
    
    /**
     * fhrt das verschlsseln aus
     */
    private String verschluessele(String text, boolean sollDecodieren){
        text = entferneSonderzeichen(text.toUpperCase());
        StringBuffer temp = new StringBuffer();
        for (int i=0; i<text.length(); i++){
            int buchstabe = text.charAt(i);
            int schluessel = key.charAt(i % keyLength);
            if (sollDecodieren)schluessel=-schluessel;
            buchstabe = ((buchstabe+schluessel+26) % 26)+65;
            temp.append((char) buchstabe);
        }
        return temp.toString();
    }
    
    /**
     * entfernt die Sonderzeichen aus dem Text.
     */
    private String entferneSonderzeichen(String text){
        StringBuffer temp = new StringBuffer();
        for (int i=0; i<text.length();i++)
        if (istGrossBuchstabe(text.charAt(i)))
        temp.append(text.charAt(i));
        return new String(temp);
    }
    
    /**
     * berechnet die Hufigkeiten, kappa und l nach Friedmann.
     */
    private int[] haeufigkeiten(String vonText){
        haeufigkeiten= new int[26];
        Character c;
        if (vonText!=null){
            for (int i=0; i<vonText.length();i++){
                c=vonText.charAt(i);
                if (istGrossBuchstabe(c)) haeufigkeiten[Character.getNumericValue(c)-10]++;
            }
        }
        return haeufigkeiten;
    }
    
    /**
     * berechnet zum aktuellen Codetext kappa.
     */
    private void kappa(){
        int zaehler=0;
        for (int i=0;i<26;i++){
            zaehler+=haeufigkeiten[i]*(haeufigkeiten[i]-1);
        }
        kappa=(double)zaehler/(double)(codeTextLength*(codeTextLength-1)) ;
    }
    
    /**
     * berechnet zum aktuellen Codetext die Schlsselwortlnge nach Friedmann.
     */
    private void laenge(){
        laenge=0.0377*codeTextLength/((codeTextLength-1)*kappa-0.0385*codeTextLength+0.0762);
    }
    
    /**
     * ruft die Berechnungsfunktionen auf, wenn der Codetext neu gestzt wurde.
     */
    private void analyse(){
        code=entferneSonderzeichen(code);
        codeTextLength=code.length();
        haeufigkeiten=haeufigkeiten(code);
        kappa();
        laenge();
    }

}

